@@ -23,7 +23,6 @@ import enum
 
 
 class HighlightMatchingOccurrences(object):
-
     # Register style element
     _styleElements = [
         (
@@ -54,6 +53,7 @@ class HighlightMatchingOccurrences(object):
     def _doHighlight(self, text):
         # make cursor at the beginning of the first visible block
         cursor = self.cursorForPosition(QtCore.QPoint(0, 0))
+        cursor.movePosition(cursor.StartOfWord)
         doc = self.document()
 
         color = self.getStyleElementFormat("editor.highlightMatchingOccurrences").back
@@ -76,16 +76,40 @@ class HighlightMatchingOccurrences(object):
                 continue
 
             endRect = self.cursorRect(cursor)
-            if endRect.bottom() > self.height():
+            cursor.setPosition(min(cursor.position(), cursor.anchor()))
+            startRect = self.cursorRect(cursor)
+
+            if startRect.top() > self.height():
                 # rest of document is not visible, don't bother highlighting
                 break
 
-            cursor.setPosition(min(cursor.position(), cursor.anchor()))
-            startRect = self.cursorRect(cursor)
             width = endRect.left() - startRect.left()
-            painter.drawRect(
-                startRect.left(), startRect.top(), width, startRect.height()
-            )
+            cursorHeight = startRect.height()
+
+            heightDiff = endRect.top() - startRect.top()
+            if heightDiff == 0:
+                painter.drawRect(startRect.left(), startRect.top(), width, cursorHeight)
+            elif heightDiff > 0:
+                cursor.movePosition(cursor.EndOfLine)
+                secondLineY = self.cursorRect(cursor).top()
+                endLineY = endRect.top()
+                fullLineStartX = 0
+                fullLineEndX = self.width()
+
+                # first partial line
+                width = fullLineEndX - startRect.left()
+                painter.drawRect(startRect.left(), startRect.top(), width, cursorHeight)
+
+                # full lines in between
+                if endLineY > secondLineY:
+                    width = fullLineEndX - fullLineStartX
+                    height = endLineY - secondLineY
+                    painter.drawRect(fullLineStartX, secondLineY, width, height)
+
+                # last partial line
+                width = endRect.left() - fullLineStartX
+                if width > 0:
+                    painter.drawRect(fullLineStartX, endLineY, width, cursorHeight)
 
             # move to end of word again, otherwise we never advance in the doc
             cursor.movePosition(cursor.EndOfWord)
@@ -212,7 +236,6 @@ class _MatchResult:
 
 
 class HighlightMatchingBracket(object):
-
     # Register style element
     _styleElements = [
         (
@@ -313,7 +336,7 @@ class HighlightMatchingBracket(object):
             if self.parser() is not None and self.parser().name() != ""
             else _PlainTextParenIterator
         )
-        for (paren, pos) in our_iterator(cursor, direction):
+        for paren, pos in our_iterator(cursor, direction):
             if paren in stacking:
                 stacked_paren.append((paren, pos))
             elif paren in unstacking:
@@ -500,7 +523,6 @@ class HighlightCurrentLine(object):
 
 
 class IndentationGuides(object):
-
     # Register style element
     _styleElements = [
         (
@@ -653,7 +675,6 @@ class CodeFolding(object):
 
 
 class LongLineIndicator(object):
-
     # Register style element
     _styleElements = [
         (
@@ -765,7 +786,6 @@ class ShowLineEndings(object):
 
 
 class LineNumbers(object):
-
     # Margin on both side of the line numbers
     _LineNumberAreaMargin = 3
 
@@ -1028,7 +1048,6 @@ class LineNumbers(object):
 
 
 class BreakPoints(object):
-
     _breakPointWidth = 11  # With of total bar, actual points are smaller
 
     # Register style element
@@ -1100,9 +1119,9 @@ class BreakPoints(object):
             # Get debug indicator and list of sorted breakpoints
             debugBlockIndicator = editor._debugLineIndicator - 1
             virtualBreakpoint = self._virtualBreakpoint - 1
-            blocknumbers = [i - 1 for i in sorted(self.parent()._breakPoints)]
+            breakpoints = self.parent()._breakPoints
             if not (
-                blocknumbers
+                len(breakpoints) > 0
                 or editor._debugLineIndicator
                 or editor._debugLineIndicators
                 or virtualBreakpoint > 0
@@ -1122,16 +1141,27 @@ class BreakPoints(object):
             painter.setRenderHint(painter.Antialiasing)
 
             # Draw breakpoints
-            for blockNumber in blocknumbers:
+            for linenr, (enabled, *_) in sorted(breakpoints.items()):
+                blockNumber = linenr - 1
                 if blockNumber < startBlockNumber:
                     continue
                 # Get block
                 block = editor.document().findBlockByNumber(blockNumber)
                 if block.isValid():
                     y = editor.blockBoundingGeometry(block).y() + bulletOffset
-                    painter.drawEllipse(
-                        int(margin), int(y), int(bulletWidth), int(bulletWidth)
-                    )
+                    if enabled:
+                        painter.drawEllipse(
+                            int(margin), int(y), int(bulletWidth), int(bulletWidth)
+                        )
+                    else:
+                        painter.drawPie(
+                            int(margin),
+                            int(y),
+                            int(bulletWidth),
+                            int(bulletWidth),
+                            90 * 16,
+                            180 * 16,
+                        )
 
             # Draw *the* debug marker
             if debugBlockIndicator > 0:
@@ -1178,7 +1208,7 @@ class BreakPoints(object):
         # Create widget that draws the breakpoints
         self.__breakPointArea = self.__BreakPointArea(self)
         self.addLeftMargin(BreakPoints, self.getBreakPointAreaWidth)
-        self._breakPoints = {}  # int -> block
+        self._breakPoints = {}  # int -> enabled, block, blockPrev, blockNext
         self._debugLineIndicator = 0
         self._debugLineIndicators = set()
         self.blockCountChanged.connect(self.__onBlockCountChanged)
@@ -1188,9 +1218,38 @@ class BreakPoints(object):
         above.
         """
         newBreakPoints = {}
+        breakPointDeleted = False
 
         for linenr in list(self._breakPoints):
-            block, block_previous, block_next = self._breakPoints[linenr]
+            enabled, block, block_previous, block_next = self._breakPoints[linenr]
+
+            # Apparently there is a bug in Qt5 and Qt6 that can cause a segmentation fault.
+            # When there is a faulty block "block.previous()", calling methods such as
+            # "block.previous().blockNumber()" will crash Pyzo immediately.
+            # These crashes happend sometimes after Qt re-created a block and we still
+            # have the outdated block reference in our self._breakPoints dictionary.
+            # Such a block re-creation will happen for example during undo operations, but
+            # also when a block is merged with a previous empty one (by pressing backspace
+            # at the beginning of a line below an empty line).
+            # See https://github.com/pyzo/pyzo/pull/949
+            #
+            # So, to avoid crashes, we discard breakpoints that had their block re-created.
+            # To detect if the block was re-created, we check if its userData was reset
+            # to None. When adding a block to "self._breakPoints", we make sure it has
+            # userData set to something different than None.
+            #
+            # According to the Qt docs from https://doc.qt.io/qt-6/qtextblock.html:
+            # "The user data object is not stored in the undo history, so it will not be
+            # available after undoing the deletion of a text block."
+
+            if block.userData() is None:
+                # The block assigned to the breakpoint was re-created by Qt after we
+                # added it to the self._breakPoints dict.
+                # To avoid a crash we delete the breakpoint.
+                del self._breakPoints[linenr]
+                breakPointDeleted = True
+                continue
+
             block_linenr = block.blockNumber() + 1
             prev_ok = block.previous().blockNumber() == block_previous.blockNumber()
             next_ok = block.next().blockNumber() == block_next.blockNumber()
@@ -1202,51 +1261,103 @@ class BreakPoints(object):
                     else:
                         # Update refs
                         self._breakPoints[linenr] = (
+                            enabled,
                             block,
                             block.previous(),
                             block.next(),
                         )
                 else:
-                    # Update linenr - this is the only case where "move" th bp
+                    # Update linenr -- this is the only case where we "move" the breakpoint
                     newBreakPoints[block_linenr] = self._breakPoints.pop(linenr)
+                    breakPointDeleted = True
             else:
                 if block_linenr == linenr:
                     # Just update refs
-                    self._breakPoints[linenr] = block, block.previous(), block.next()
+                    self._breakPoints[linenr] = (
+                        enabled,
+                        block,
+                        block.previous(),
+                        block.next(),
+                    )
                 else:
-                    # Delete breakpoint? Meh, just update refs
-                    self._breakPoints[linenr] = block, block.previous(), block.next()
+                    # unexpected --> delete breakpoint
+                    del self._breakPoints[linenr]
+                    breakPointDeleted = True
 
-        if newBreakPoints:
+        if newBreakPoints or breakPointDeleted:
             self._breakPoints.update(newBreakPoints)
             self.breakPointsChanged.emit(self)
             self.__breakPointArea.update()
 
-    def breakPoints(self):
+    def breakPoints(self, triState=False):
         """A list of breakpoints for this editor."""
-        return list(sorted(self._breakPoints))
+        if triState:
+            return sorted(
+                (linenr, enabled) for linenr, (enabled, *_) in self._breakPoints.items()
+            )
+        else:
+            return sorted(
+                linenr for linenr, (enabled, *_) in self._breakPoints.items() if enabled
+            )
 
     def clearBreakPoints(self):
         """Remove all breakpoints for this editor."""
         for linenr in self.breakPoints():
-            self.toggleBreakpoint(linenr)
+            self.toggleBreakpoint(linenr, triState=False)
 
-    def toggleBreakpoint(self, linenr=None):
-        """Turn breakpoint on/off for given linenr of current line."""
+    def toggleBreakpoint(self, linenr=None, triState=True):
+        """Turn breakpoint on / half (disabled) / off (removed) for given linenr of current line."""
         if linenr is None:
             linenr = self.textCursor().blockNumber() + 1
+
         if linenr in self._breakPoints:
-            self._breakPoints.pop(linenr)
+            enabled, b, bPrev, bNext = self._breakPoints.pop(linenr)
+            if triState and enabled:
+                enabled = False
+            else:
+                b = None  # indicate to not add a new modified breakpoint
         else:
+            enabled = True
             c = self.textCursor()
             c.movePosition(c.Start)
             c.movePosition(c.NextBlock, c.MoveAnchor, linenr - 1)
             b = c.block()
-            self._breakPoints[linenr] = b, b.previous(), b.next()
+            bPrev = b.previous()
+            bNext = b.next()
+
+        if b is not None:
+            # As described in method "__onBlockCountChanged", we want to make sure that
+            # the userData of the block is set to anything but None. It is totally ok that
+            # other Pyzo modules such as the highlighter will overwrite that userData as
+            # long as it is not set back to None.
+            if b.userData() is None:
+                b.setUserData(
+                    QtGui.QTextBlockUserData()
+                )  # just something other than None
+
+            self._breakPoints[linenr] = enabled, b, bPrev, bNext
 
         self.breakPointsChanged.emit(self)
         self.__breakPointArea.update()
 
+    def jumpPreviousBreakpoint(self):
+        currentLinenr = self.textCursor().blockNumber() + 1
+        newLinenr = max(
+            (linenr for linenr in self._breakPoints if linenr < currentLinenr),
+            default=None,
+        )
+        if newLinenr is not None:
+            self.gotoLine(newLinenr, keepHorizontalPos=True)
+
+    def jumpNextBreakpoint(self):
+        currentLinenr = self.textCursor().blockNumber() + 1
+        newLinenr = min(
+            (linenr for linenr in self._breakPoints if linenr > currentLinenr),
+            default=None,
+        )
+        if newLinenr is not None:
+            self.gotoLine(newLinenr, keepHorizontalPos=True)
+
     def setDebugLineIndicator(self, linenr, active=True):
         """Set the debug line indicator to the given line number.
         If None or 0, the indicator is hidden.
