@@ -8,13 +8,13 @@
 Code editor extensions that provides autocompleter functionality
 """
 
-import pyzo
 from ..qt import QtGui, QtCore, QtWidgets
 
 Qt = QtCore.Qt
 
 import keyword
 
+
 # TODO: use this CompletionListModel to style the completion suggestions (class names, method names, keywords etc)
 class CompletionListModel(QtCore.QStringListModel):
     def data(self, index, role):
@@ -99,6 +99,13 @@ class AutoCompletion(object):
         """
         self.__popupSize = width, height
 
+    def setAutocompleteCaseSensitive(self, b):
+        """
+        Set the case sensitivity for autocompletion.
+        """
+        cs = Qt.CaseSensitive if b else Qt.CaseInsensitive
+        self.__completer.setCaseSensitivity(cs)
+
     def setAutocompleteMinChars(self, n):
         """
         Set the number of chars where we show the popup.
@@ -131,7 +138,6 @@ class AutoCompletion(object):
             or startcursor.position() != self.__autocompleteStart.position()
             or not self.autocompleteVisible()
         ):
-
             self.__autocompleteStart = startcursor
             self.__autocompleteStart.setKeepPositionOnInsert(True)
 
@@ -199,15 +205,17 @@ class AutoCompletion(object):
         # Initial choice for position of the completer
         position = self.cursorRect(cur).bottomLeft() + self.viewport().pos()
 
-        # Check if the completer is going to go off the screen
-        # top_geometry = QtWidgets.qApp.qApp.screens()[0].availableVirtualSize()
-        top_geometry = pyzo.main.geometry()
-        global_position = self.mapToGlobal(position)
-        if global_position.y() + geometry.height() > top_geometry.height():
+        # Check if the completer is going to go off the editor/shell widget
+        if (
+            position.y() + geometry.height()
+            > self.viewport().pos().y() + self.viewport().height()
+        ):
             # Move the completer to above the current line
             position = self.cursorRect(cur).topLeft() + self.viewport().pos()
             global_position = self.mapToGlobal(position)
             global_position -= QtCore.QPoint(0, int(geometry.height()))
+        else:
+            global_position = self.mapToGlobal(position)
 
         self.__completer.popup().move(global_position)
 
