@@ -39,11 +39,11 @@ class CharToken(Token):
 tokenProg = re.compile(
     "(["
     + ALPHANUM
-    + "_]+)|"
-    + "(\/\/)|"  # Identifiers/numbers (group 1) or
-    + "(\/\*)|"  # Single line comment (group 2)
-    + "('\\\\?.')|"  # Comment (group 3) or
-    + '(")'  # char (group 4)  # string (group 5)
+    + "_]+)|"  # Identifiers/numbers (group 1) or
+    + "(//)|"  # Single line comment (group 2)
+    + "(/\\*)|"  # Comment (group 3) or
+    + "('\\\\?.')|"  # char (group 4)
+    + '(")'  # string (group 5)
 )
 
 
@@ -100,7 +100,6 @@ class CParser(Parser):
 
         # Enter the main loop that iterates over the tokens and skips strings
         while True:
-
             # Get next tokens
             tokens = self._findNextToken(line, pos)
             if not tokens:
