@@ -82,7 +82,7 @@ class Parser(threading.Thread):
         # Lock to enable save threading
         self._lock = threading.RLock()
 
-        # Set deamon
+        # Set daemon
         self.daemon = True
         self._exit = False
 
@@ -323,7 +323,6 @@ class Parser(threading.Thread):
                     return
 
                 if self._job:
-
                     # Savely obtain job
                     self._lock.acquire()
                     job = self._job
@@ -394,7 +393,6 @@ class Parser(threading.Thread):
         # Find objects!
         # type can be: cell, class, def, import, var
         for i in range(len(lines)):
-
             # Obtain line
             line = lines[i]
 
@@ -430,7 +428,8 @@ class Parser(threading.Thread):
             line, cmnt = line.rstrip(), cmnt.lower().strip()
 
             # Detect todos
-            if cmnt and (cmnt.startswith("todo:") or cmnt.startswith("2do:")):
+            firstWord = cmnt.lstrip().split(" ", 1)[0].rstrip(":")
+            if firstWord.lower() in ["todo", "2do", "fixme"]:
                 item = FictiveObject("todo", i, indent, cmnt)
                 item.linenr2 = i + 1  # a todo is active at one line only
                 leafs.append(item)
