@@ -45,7 +45,6 @@ def determineEncoding(bb):
     encoding = "UTF-8"
 
     for line in firstTwoLines:
-
         # Try to make line a string
         try:
             line = line.decode("ASCII").strip()
@@ -54,12 +53,10 @@ def determineEncoding(bb):
 
         # Has comment?
         if line and line[0] == "#":
-
             # Matches regular expression given in PEP 0263?
             expression = r"coding[:=]\s*([-\w.]+)"
             result = re.search(expression, line)
             if result:
-
                 # Is it a known encoding? Correct name if it is
                 candidate_encoding = result.group(1)
                 try:
@@ -184,7 +181,6 @@ def createEditor(parent, filename=None):
     """
 
     if filename is None:
-
         # Increase counter
         global newFileCounter
         newFileCounter += 1
@@ -198,7 +194,6 @@ def createEditor(parent, filename=None):
         editor._name = "<tmp {}>".format(newFileCounter)
 
     else:
-
         # check and normalize
         if not os.path.isfile(filename):
             raise IOError("File does not exist '%s'." % filename)
@@ -255,7 +250,6 @@ def createEditor(parent, filename=None):
 
 
 class PyzoEditor(BaseTextCtrl):
-
     # called when dirty changed or filename changed, etc
     somethingChanged = QtCore.Signal()
 
@@ -404,7 +398,6 @@ class PyzoEditor(BaseTextCtrl):
         # test the modification time...
         mtime = os.path.getmtime(path)
         if mtime != self._modifyTime:
-
             # ask user
             dlg = QtWidgets.QMessageBox(self)
             dlg.setWindowTitle("File was changed")
@@ -539,11 +532,8 @@ class PyzoEditor(BaseTextCtrl):
         bb = text.encode(self.encoding)
 
         # Store
-        f = open(filename, "wb")
-        try:
+        with open(filename, "wb") as f:
             f.write(bb)
-        finally:
-            f.close()
 
         # Update stats
         self._filename = normalizePath(filename)
@@ -612,9 +602,12 @@ class PyzoEditor(BaseTextCtrl):
         cursor.setPosition(start)
         cursor.movePosition(cursor.StartOfBlock)
         cursor.setPosition(end, cursor.KeepAnchor)
-        cursor.movePosition(cursor.NextBlock, cursor.KeepAnchor)
-
-        text = cursor.selectedText()
+        notAtLastBlock = cursor.movePosition(cursor.NextBlock, cursor.KeepAnchor)
+        if notAtLastBlock:
+            text = cursor.selectedText()
+        else:
+            cursor.movePosition(cursor.End, cursor.KeepAnchor)
+            text = cursor.selectedText() + "\u2029"
         cursor.setPosition(start)
         cursor.movePosition(cursor.StartOfBlock)
         cursor.insertText(text)
@@ -785,6 +778,10 @@ class PyzoEditor(BaseTextCtrl):
                     if fictiveClass:
                         aco.addNames(fictiveClass.members)
                         classNames.extend(fictiveClass.supers)
+                        if className in classNames:
+                            # avoid infinite loop if the parent class is the same as the
+                            # current class, see https://github.com/pyzo/pyzo/issues/944
+                            break
                 else:
                     nameForShell = className
                     break
