@@ -155,7 +155,6 @@ class FileItem:
     """
 
     def __init__(self, editor):
-
         # Store editor
         self._editor = editor
 
@@ -211,6 +210,8 @@ class FindReplaceWidget(QtWidgets.QFrame):
         # init layout
         layout = QtWidgets.QHBoxLayout(self)
         layout.setSpacing(0)
+        margin = 0
+        layout.setContentsMargins(margin, margin, margin, margin)
         self.setLayout(layout)
 
         # Create some widgets first to realize a correct tab order
@@ -236,7 +237,6 @@ class FindReplaceWidget(QtWidgets.QFrame):
         layout.addSpacing(10)
 
         if True:
-
             # Create sub layouts
             vsubLayout = QtWidgets.QVBoxLayout()
             hsubLayout = QtWidgets.QHBoxLayout()
@@ -273,7 +273,6 @@ class FindReplaceWidget(QtWidgets.QFrame):
         layout.addSpacing(10)
 
         if True:
-
             # Create sub layouts
             vsubLayout = QtWidgets.QVBoxLayout()
             hsubLayout = QtWidgets.QHBoxLayout()
@@ -306,7 +305,6 @@ class FindReplaceWidget(QtWidgets.QFrame):
         layout.addSpacing(10)
 
         if True:
-
             # Create sub layouts
             vsubLayout = QtWidgets.QVBoxLayout()
             vsubLayout.setSpacing(0)
@@ -325,7 +323,6 @@ class FindReplaceWidget(QtWidgets.QFrame):
             vsubLayout.addWidget(self._regExp, 0)
 
         if True:
-
             # Create sub layouts
             vsubLayout = QtWidgets.QVBoxLayout()
             vsubLayout.setSpacing(0)
@@ -521,27 +518,22 @@ class FindReplaceWidget(QtWidgets.QFrame):
 
         # get text to find
         needle = self._findText.text()
+
+        QRE = QtCore.QRegularExpression
+        PatternOption = QRE.PatternOption
+        regexFlags = PatternOption.NoPatternOption
+        if not self._caseCheck.isChecked():
+            regexFlags |= PatternOption.CaseInsensitiveOption
+
         if self._regExp.isChecked():
-            # Make needle a QRegExp; speciffy case-sensitivity here since the
-            # FindCaseSensitively flag is ignored when finding using a QRegExp
-            needle = QtCore.QRegExp(
-                needle,
-                QtCore.Qt.CaseSensitive
-                if self._caseCheck.isChecked()
-                else QtCore.Qt.CaseInsensitive,
-            )
+            needle = QRE(needle, regexFlags)
         elif self._wholeWord.isChecked():
-            # Use regexp, because the default begaviour does not find
+            # Use regexp, because the default behaviour does not find
             # whole words correctly, see issue #276
             # it should *not* find this in this_word
-            needle = QtCore.QRegExp(
-                r"\b" + needle + r"\b",
-                QtCore.Qt.CaseSensitive
-                if self._caseCheck.isChecked()
-                else QtCore.Qt.CaseInsensitive,
-            )
+            needle = QRE(r"\b" + QRE.escape(needle) + r"\b", regexFlags)
 
-        # estblish start position
+        # establish start position
         cursor = editor.textCursor()
         result = editor.document().find(needle, cursor, flags)
 
@@ -644,7 +636,6 @@ class FindReplaceWidget(QtWidgets.QFrame):
         editor.setTextCursor(originalPosition)
 
     def replaceInAllFiles(self, event=None):
-
         for editor in pyzo.editors:
             self.replaceAll(event, editor)
 
@@ -656,6 +647,9 @@ class FileTabWidget(CompactTabWidget):
 
     """
 
+    # Signal to indicate that file tabs were added, updated or removed
+    fileTabsChanged = QtCore.Signal()
+
     def __init__(self, parent):
         CompactTabWidget.__init__(self, parent, padding=(2, 1, 0, 4))
 
@@ -679,7 +673,6 @@ class FileTabWidget(CompactTabWidget):
         self.setTitleInMainWindowWhenTabChanged(-1)
 
     def setTitleInMainWindowWhenTabChanged(self, index):
-
         # Valid index?
         if index < 0 or index >= self.count():
             pyzo.main.setMainTitle()  # No open file
@@ -759,7 +752,6 @@ class FileTabWidget(CompactTabWidget):
             self.setCurrentIndex(item)
 
         elif isinstance(item, FileItem):
-
             items = self.items()
             for i in range(self.count()):
                 if item is items[i]:
@@ -767,7 +759,6 @@ class FileTabWidget(CompactTabWidget):
                     break
 
         elif isinstance(item, str):
-
             items = self.items()
             for i in range(self.count()):
                 if item == items[i].filename:
@@ -841,7 +832,6 @@ class FileTabWidget(CompactTabWidget):
             )
 
         if theIndex >= 0:
-
             # Close tab
             CompactTabWidget.removeTab(self, theIndex)
 
@@ -849,6 +839,8 @@ class FileTabWidget(CompactTabWidget):
             items[theIndex].editor.destroy()
             gc.collect()
 
+            self.fileTabsChanged.emit()
+
     def addItem(self, item, update=True):
         """addItem(item, update=True)
 
@@ -910,7 +902,6 @@ class FileTabWidget(CompactTabWidget):
             xx.append(item)
 
         for i in range(len(items)):
-
             # Get item
             item = items[i]
             if item is None:
@@ -942,11 +933,12 @@ class FileTabWidget(CompactTabWidget):
 
             # Determine text color. Is main file? Is current?
             if self._mainFile == item.id:
-                tabBar.setTabTextColor(i, QtGui.QColor("#008"))
+                color = "#0cf" if pyzo.darkQt else "#008"
             elif i == self.currentIndex():
-                tabBar.setTabTextColor(i, QtGui.QColor("#000"))
+                color = "#fff" if pyzo.darkQt else "#000"
             else:
-                tabBar.setTabTextColor(i, QtGui.QColor("#444"))
+                color = "#ddd" if pyzo.darkQt else "#444"
+            tabBar.setTabTextColor(i, QtGui.QColor(color))
 
             # Get number of blocks
             nBlocks = item.editor.blockCount()
@@ -957,6 +949,8 @@ class FileTabWidget(CompactTabWidget):
             but = tabBar.tabButton(i, QtWidgets.QTabBar.LeftSide)
             but.updateIcon(item.dirty, self._mainFile == item.id, item.pinned, nBlocks)
 
+        self.fileTabsChanged.emit()
+
 
 class EditorTabs(QtWidgets.QWidget):
     """The EditorTabs instance manages the open files and corresponding
@@ -1013,9 +1007,16 @@ class EditorTabs(QtWidgets.QWidget):
         # restore state (call later so that the menu module can bind to the
         # currentChanged signal first, in order to set tab/indentation
         # checkmarks appropriately)
-        # todo: Resetting the scrolling would work better if set after
-        # the widgets are properly sized.
-        pyzo.callLater(self.restoreEditorState)
+
+        # self.restoreEditorState should be called from outside after the paintNow call
+        # otherwise the horizontal scrollbar would be set for a too small widget size
+
+    @property
+    def _fileDialogOptions(self):
+        options = QtWidgets.QFileDialog.Option(0)
+        if not pyzo.config.advanced.useNativeFileDialogs:
+            options |= QtWidgets.QFileDialog.Option.DontUseNativeDialog
+        return options
 
     def addContextMenu(self):
         """Adds a context menu to the tab bar"""
@@ -1167,7 +1168,7 @@ class EditorTabs(QtWidgets.QWidget):
         filter += "All (*)"
         if True:
             filenames = QtWidgets.QFileDialog.getOpenFileNames(
-                self, msg, startdir, filter
+                self, msg, startdir, filter, options=self._fileDialogOptions
             )
             if isinstance(filenames, tuple):  # PySide
                 filenames = filenames[0]
@@ -1175,7 +1176,9 @@ class EditorTabs(QtWidgets.QWidget):
             # Example how to preselect files, can be used when the users
             # opens a file in a project to select all files currently not
             # loaded.
-            d = QtWidgets.QFileDialog(self, msg, startdir, filter)
+            d = QtWidgets.QFileDialog(
+                self, msg, startdir, filter, options=self._fileDialogOptions
+            )
             d.setFileMode(d.ExistingFiles)
             d.selectFile('"codeparser.py" "editorStack.py"')
             d.exec_()
@@ -1206,7 +1209,9 @@ class EditorTabs(QtWidgets.QWidget):
 
         # show dialog
         msg = "Select a directory to open"
-        dirname = QtWidgets.QFileDialog.getExistingDirectory(self, msg, startdir)
+        dirname = QtWidgets.QFileDialog.getExistingDirectory(
+            self, msg, startdir, options=self._fileDialogOptions
+        )
 
         # was a dir selected?
         if not dirname:
@@ -1215,7 +1220,7 @@ class EditorTabs(QtWidgets.QWidget):
         # load
         self.loadDir(dirname)
 
-    def loadFile(self, filename, updateTabs=True):
+    def loadFile(self, filename, updateTabs=True, ignoreFail=False):
         """Load the specified file.
         On success returns the item of the file, also if it was
         already open."""
@@ -1248,11 +1253,12 @@ class EditorTabs(QtWidgets.QWidget):
             # Notify in logger
             print("Error loading file: ", err)
             # Make sure the user knows
-            m = QtWidgets.QMessageBox(self)
-            m.setWindowTitle("Error loading file")
-            m.setText(str(err))
-            m.setIcon(m.Warning)
-            m.exec_()
+            if not ignoreFail:
+                m = QtWidgets.QMessageBox(self)
+                m.setWindowTitle("Error loading file")
+                m.setText(str(err))
+                m.setIcon(m.Warning)
+                m.exec_()
             return None
 
         # create list item
@@ -1339,7 +1345,9 @@ class EditorTabs(QtWidgets.QWidget):
         filter += "C (*.c *.h *.cpp);;"
         # filter += "Py+Cy+C (*.py *.pyw *.pyi *.pyx *.pxd *.c *.h *.cpp);;"
         filter += "All (*.*)"
-        filename = QtWidgets.QFileDialog.getSaveFileName(self, msg, startdir, filter)
+        filename = QtWidgets.QFileDialog.getSaveFileName(
+            self, msg, startdir, filter, options=self._fileDialogOptions
+        )
         if isinstance(filename, tuple):  # PySide
             filename = filename[0]
 
@@ -1442,7 +1450,6 @@ class EditorTabs(QtWidgets.QWidget):
 
         # should we ask to save the file?
         if editor.document().isModified():
-
             # Ask user what to do
             close_txt, discard_txt, cancel_txt, save_txt = self._get_action_texts()
             result = simpleDialog(
@@ -1527,6 +1534,7 @@ class EditorTabs(QtWidgets.QWidget):
         pyzo.config.state.find_matchCase = fr._caseCheck.isChecked()
         pyzo.config.state.find_regExp = fr._regExp.isChecked()
         pyzo.config.state.find_wholeWord = fr._wholeWord.isChecked()
+        pyzo.config.state.find_autoHide = fr._autoHide.isChecked()
         pyzo.config.state.find_show = fr.isVisible()
         #
         pyzo.config.state.editorState2 = self._getCurrentOpenFilesAsSsdfList()
@@ -1541,6 +1549,8 @@ class EditorTabs(QtWidgets.QWidget):
                 self.newFile()
         else:
             self.newFile()
+        self._tabs.updateItems()  # without this, a single restored editor will not have
+        # correct PINNED/MAINFILE symbols
 
         # The find/replace state is set in the corresponding class during init
 
@@ -1556,7 +1566,6 @@ class EditorTabs(QtWidgets.QWidget):
 
         # Get items
         for item in self._tabs.items():
-
             # Get editor
             ed = item.editor
             if not ed._filename:
@@ -1608,10 +1617,10 @@ class EditorTabs(QtWidgets.QWidget):
                         self._tabs.setCurrentItem(fileItems[fname])
                 elif fname:
                     # a file item, create editor-item and store
-                    itm = self.loadFile(fname)
-                    fileItems[fname] = itm
+                    itm = self.loadFile(fname, ignoreFail=True)
                     # set position
                     if itm:
+                        fileItems[fname] = itm
                         try:
                             ed = itm.editor
                             cursor = ed.textCursor()
