@@ -12,7 +12,7 @@ that show information to the user in a very effective, yet subtle manner.
 
 """
 
-from pyzo.util.qt import QtCore, QtGui, QtWidgets
+from pyzo.qt import QtCore, QtGui, QtWidgets
 import pyzo
 
 
@@ -25,9 +25,9 @@ class IconArtist:
 
     """
 
-    def __init__(self, icon=None):
-
+    def __init__(self, icon=None, size=16):
         # Get pixmap from given icon (None creates empty pixmap)
+        self._wh = size
         self._pm = self._getPixmap(icon)
 
         # Instantiate painter for the pixmap
@@ -42,14 +42,13 @@ class IconArtist:
         return QtGui.QIcon(self._pm)
 
     def _getPixmap(self, icon):
-
         # Get icon if given by name
         if isinstance(icon, str):
             icon = pyzo.icons[icon]
 
         # Create pixmap
         if icon is None:
-            pm = QtGui.QPixmap(16, 16)
+            pm = QtGui.QPixmap(self._wh, self._wh)
             pm.fill(QtGui.QColor(0, 0, 0, 0))
             return pm
         elif isinstance(icon, tuple):
@@ -59,22 +58,31 @@ class IconArtist:
         elif isinstance(icon, QtGui.QPixmap):
             return icon
         elif isinstance(icon, QtGui.QIcon):
-            return icon.pixmap(16, 16)
+            return icon.pixmap(self._wh, self._wh)
         else:
             raise ValueError("Icon for IconArtis should be icon, pixmap or name.")
 
+    @staticmethod
+    def _toQColor(color):
+        return QtGui.QColor(*color) if isinstance(color, tuple) else QtGui.QColor(color)
+
     def setPenColor(self, color):
         """setPenColor(color)
         Set the color of the pen. Color can be anything that can be passed to
         Qcolor().
         """
         pen = QtGui.QPen()
-        if isinstance(color, tuple):
-            pen.setColor(QtGui.QColor(*color))
-        else:
-            pen.setColor(QtGui.QColor(color))
+        pen.setColor(self._toQColor(color))
         self._painter.setPen(pen)
 
+    def setBrushColor(self, *color):
+        """setBrushColor(color)
+        Set the color of the brush. Color can be anything that can be passed to
+        Qcolor().
+        """
+        brush = QtGui.QBrush(self._toQColor(color), QtCore.Qt.SolidPattern)
+        self._painter.setBrush(brush)
+
     def addLayer(self, overlay, x=0, y=0):
         """addOverlay(overlay, x=0, y=0)
         Add an overlay icon to the icon (add the specified position).
@@ -82,6 +90,13 @@ class IconArtist:
         pm = self._getPixmap(overlay)
         self._painter.drawPixmap(x, y, pm)
 
+    def addPolygon(self, pointList):
+        """addPolygon(pointList)
+        Add a filled polygon to the icon.
+        """
+        poly = QtGui.QPolygon([QtCore.QPoint(x, y) for x, y in pointList])
+        self._painter.drawPolygon(poly, QtCore.Qt.OddEvenFill)
+
     def addLine(self, x1, y1, x2, y2):
         """addLine( x1, y1, x2, y2)
         Add a line to the icon.
@@ -99,6 +114,8 @@ class IconArtist:
         Adds a menu arrow to the icon to let the user know the icon
         is clickable.
         """
+        if self._wh != 16:
+            raise NotImplementedError()
         x, y = 0, 12
         a1, a2 = int(strength / 2), strength
         # Zeroth line of 3+2
@@ -283,7 +300,6 @@ class ToolButtonWithMenuIndication(QtWidgets.QToolButton):
         self._menuPressed = False
 
     def setIcon(self, icon=None):
-
         # Store icon if given, otherwise use buffered version
         if icon is not None:
             self._icon = icon
@@ -393,7 +409,6 @@ class TabToolButtonWithCloseButton(TabToolButton):
             self.setIcon()
 
     def setIcon(self, icon=None):
-
         # Store icon if given, otherwise use buffered version
         if icon is not None:
             self._icon = icon
@@ -417,9 +432,13 @@ class TabToolButtonWithCloseButton(TabToolButton):
         return artist.finish().pixmap(16, 16)
 
     def _createCrossPixmap(self, alpha):
+        if pyzo.darkQt:
+            r, g, b = 255, 255, 255
+        else:
+            r, g, b = 0, 0, 0
         artist = IconArtist((5, 5))
         #
-        artist.setPenColor((0, 0, 0, alpha))
+        artist.setPenColor((r, g, b, alpha))
         #
         artist.addPoint(0, 0)
         artist.addPoint(1, 1)
@@ -431,7 +450,7 @@ class TabToolButtonWithCloseButton(TabToolButton):
         artist.addPoint(3, 1)
         artist.addPoint(4, 0)
         #
-        artist.setPenColor((0, 0, 0, int(0.5 * alpha)))
+        artist.setPenColor((r, g, b, int(0.5 * alpha)))
         #
         artist.addPoint(1, 0)
         artist.addPoint(0, 1)
@@ -453,7 +472,8 @@ class TabToolButtonWithCloseButton(TabToolButton):
         if hasattr(self, "_cross1"):
             pm = self._cross1
         else:
-            pm = self._createCrossPixmap(50)
+            alpha = 150 if pyzo.darkQt else 50
+            pm = self._createCrossPixmap(alpha)
         return pm
 
     def getCrossPixmap2(self):
@@ -471,7 +491,6 @@ class EditorTabToolButton(TabToolButtonWithCloseButton):
     """
 
     def updateIcon(self, isDirty, isMain, isPinned, nBlocks=10001):
-
         # Init drawing
         artist = IconArtist()
 
@@ -497,7 +516,7 @@ class EditorTabToolButton(TabToolButtonWithCloseButton):
         else:
             nLines = 5
         #
-        fraction = float(nBlocks) / 10 ** nLines
+        fraction = float(nBlocks) / 10**nLines
         fraction = min(fraction, 1.0)
         #
         for i in range(nLines):
@@ -550,7 +569,6 @@ class ShellIconMaker:
     MAX_ITERS_IN_LEVEL_1 = 2
 
     def __init__(self, objectWithIcon):
-
         self._objectWithIcon = objectWithIcon
 
         # Motion properties
@@ -564,7 +582,7 @@ class ShellIconMaker:
 
         # Create timer
         self._timer = QtCore.QTimer(None)
-        self._timer.setInterval(150)
+        self._timer.setInterval(200)
         self._timer.setSingleShot(False)
         self._timer.timeout.connect(self.onTimer)
 
@@ -572,7 +590,6 @@ class ShellIconMaker:
         self._objectWithIcon.setIcon(icon)
 
     def _createBlobPixmap(self):
-
         artist = IconArtist()
         artist.setPenColor((0, 150, 0, 255))
         artist.addPoint(1, 1)
