@@ -181,6 +181,7 @@ def getEnvFromKernelInfo(info):
     #
     env.pop("TK_LIBRARY", "")
     env.pop("TCL_LIBRARY", "")
+    env.pop("QT_MAC_WANTS_LAYER")
     env["PYTHONPATH"] = pythonPath
     # Jython does not use PYTHONPATH but JYTHONPATH
     env["JYTHONPATH"] = pyzo.pyzoDir + os.pathsep + os.environ.get("JYTHONPATH", "")
@@ -329,7 +330,6 @@ class KernelBroker:
         self._streamReader = None
 
         if destroy:
-
             # Stop timer
             self._timer.unbind(self.mainLoopIter)
             self._timer.stop()
@@ -396,31 +396,37 @@ class KernelBroker:
         command = getCommandFromKernelInfo(self._info, self._kernelCon.port1)
         env = getEnvFromKernelInfo(self._info)
 
-        # Wrap command in call to 'cmd'?
-        if sys.platform.startswith("win"):
-            # as the author from Pype writes:
-            # if we don't run via a command shell, then either sometimes we
-            # don't get wx GUIs, or sometimes we can't kill the subprocesses.
-            # And I also see problems with Tk.
-            # But we only use it if we are sure that cmd is available.
-            # See pyzo issue #240
-            try:
-                subprocess.check_output('cmd /c "cd"', shell=True)
-            except (IOError, subprocess.SubprocessError):
-                pass  # Do not use cmd
-            else:
-                command = 'cmd /c "{}"'.format(command)
+        try:
+            # Wrap command in call to 'cmd'?
+            if sys.platform.startswith("win"):
+                # as the author from Pype writes:
+                # if we don't run via a command shell, then either sometimes we
+                # don't get wx GUIs, or sometimes we can't kill the subprocesses.
+                # And I also see problems with Tk.
+                # But we only use it if we are sure that cmd is available.
+                # See pyzo issue #240
+                try:
+                    subprocess.check_output('cmd /c "cd"', shell=True)
+                except (IOError, subprocess.SubprocessError):
+                    pass  # Do not use cmd
+                else:
+                    command = 'cmd /c "{}"'.format(command)
 
-        # Start process
-        self._process = subprocess.Popen(
-            command,
-            shell=True,
-            env=env,
-            cwd=cwd,
-            stdin=subprocess.PIPE,  # Fixes issue 165
-            stdout=subprocess.PIPE,
-            stderr=subprocess.STDOUT,
-        )
+            # Start process
+            self._process = subprocess.Popen(
+                command,
+                shell=True,
+                env=env,
+                cwd=cwd,
+                stdin=subprocess.PIPE,  # Fixes issue 165
+                stdout=subprocess.PIPE,
+                stderr=subprocess.STDOUT,
+            )
+        except Exception as err:
+            self._pending_restart = None
+            self._process = None
+            self._onKernelDied(str(err))
+            return
 
         # Set timeout for connection, i.e. after how much time of
         # unresponsive ness is the kernel found to be running extension code
@@ -486,7 +492,7 @@ class KernelBroker:
         if not self._terminator:
             self.terminate("because connecton was lost", "KILL", 0.5)
 
-    def _onKernelDied(self, returncode=0):
+    def _onKernelDied(self, why=None):
         """_onKernelDied()
 
         Kernel process died. Clean up!
@@ -495,7 +501,7 @@ class KernelBroker:
 
         # If the kernel did not start yet, probably the command is invalid
         if self._kernelCon and self._kernelCon.is_waiting:
-            msg = "The process failed to start (invalid command?)."
+            msg = "The process failed to start."
         elif not self.isTerminating():
             msg = "Kernel process exited."
         elif not self._terminator._prev_action:
@@ -510,8 +516,11 @@ class KernelBroker:
 
         if self._context.connection_count:
             # Notify
-            returncodeMsg = "\n%s (%s)\n\n" % (msg, str(returncode))
-            self._strm_broker.send(returncodeMsg)
+            if isinstance(why, int):
+                total_msg = "\n%s (%s)\n\n" % (msg, str(why))
+            else:
+                total_msg = "\n%s\n%s\n\n" % (msg, str(why))
+            self._strm_broker.send(total_msg)
             # Empty prompt and signal dead
             self._strm_prompt.send("\b")
             self._stat_interpreter.send("Dead")
@@ -658,7 +667,6 @@ class KernelTerminator:
     """
 
     def __init__(self, broker, reason="by user", action="TERM", timeout=0.0):
-
         # Init/store
         self._broker = broker
         self._reason = reason
@@ -675,7 +683,6 @@ class KernelTerminator:
             self.next()
 
     def next(self):
-
         # Get action
         action = self._next_action
 
@@ -745,7 +752,7 @@ class StreamReader(threading.Thread):
         self._process = process
         self._strm_raw = strm_raw
         self._strm_broker = strm_broker
-        self.deamon = True
+        self.daemon = True
         self._exit = False
 
     def stop(self, timeout=1.0):
@@ -767,6 +774,8 @@ class StreamReader(threading.Thread):
             if not msg:  # or self._process.poll() is not None:
                 break
         # self._strm_broker.send('streamreader exit\n')
+        self._process.stdout.close()
+        self._process.stdin.close()
 
 
 class Kernelmanager:
@@ -783,7 +792,6 @@ class Kernelmanager:
     """
 
     def __init__(self, public=False):
-
         # Set whether other machines in this network may connect to our kernels
         self._public = public
 
@@ -817,7 +825,6 @@ class Kernelmanager:
         return port
 
     def getKernelList(self):
-
         # Get info of each kernel as an ssdf struct
         infos = []
         for kernel in self._kernels:
@@ -837,7 +844,6 @@ class Kernelmanager:
 
         """
         for kernel in [kernel for kernel in self._kernels]:
-
             # Try closing the process gently: by closing stdin
             terminator = KernelTerminator(kernel, "for closing down")
 
