@@ -83,9 +83,27 @@ class MainWindow(QtWidgets.QMainWindow):
         loadIcons()
         loadFonts()
 
-        # Set qt style and test success
+        # Set qt style, test success and detect Qt dark mode
         self.setQtStyle(None)  # None means init!
 
+        if pyzo.config.settings.theme == "":
+            # set light or dark mode when running Pyzo the first time
+            pyzo.config.settings.theme = "default_dark" if pyzo.darkQt else "default"
+
+        # detect dark syntax style (light text on dark background)
+        try:
+            theme = pyzo.themes[pyzo.config.settings.theme.lower()]["data"]
+            s = theme["editor.text"]  # e.g.: "fore:#657b83, back:#fff"
+            colors = dict(
+                [tuple(s3.strip() for s3 in s2.split(":")) for s2 in s.split(",")]
+            )
+            pyzo.darkSyntax = (
+                QtGui.QColor(colors["fore"]).lightness()
+                > QtGui.QColor(colors["back"]).lightness()
+            )
+        except KeyError:
+            pyzo.darkSyntax = False
+
         # Hold the splash screen if needed
         while time.time() < splash_timeout:
             QtWidgets.qApp.sendPostedEvents()
@@ -103,6 +121,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.restoreState()
         self.paintNow()
         self.restoreState()
+        pyzo.editors.restoreEditorState()
 
         # Present user with wizard if he/she is new.
         if False:  # pyzo.config.state.newUser:
@@ -142,7 +161,6 @@ class MainWindow(QtWidgets.QMainWindow):
             time.sleep(0.01)
 
     def _populate(self):
-
         # Delayed imports
         from pyzo.core.editorTabs import EditorTabs
         from pyzo.core.shellStack import ShellStackWidget
@@ -153,10 +171,6 @@ class MainWindow(QtWidgets.QMainWindow):
         # Instantiate tool manager
         pyzo.toolManager = ToolManager()
 
-        # Check to install conda now ...
-        # from pyzo.util.bootstrapconda import check_for_conda_env
-        # check_for_conda_env()
-
         # Instantiate and start source-code parser
         if pyzo.parser is None:
             pyzo.parser = codeparser.Parser()
@@ -334,6 +348,12 @@ class MainWindow(QtWidgets.QMainWindow):
         if qstyle:
             QtWidgets.qApp.setPalette(QtWidgets.qApp.nativePalette)
 
+        # detect dark mode for widgets (might be different than dark mode for syntax)
+        pal = QtWidgets.qApp.palette()
+        pyzo.darkQt = (
+            pal.window().color().lightness() < pal.windowText().color().lightness()
+        )
+
         # Done
         return qstyle
 
@@ -343,13 +363,6 @@ class MainWindow(QtWidgets.QMainWindow):
         # Are we restaring?
         restarting = time.time() - self._closeflag < 1.0  # noqa: F841
 
-        # Save settings
-        pyzo.saveConfig()
-        pyzo.command_history.save()
-
-        # Stop command server
-        commandline.stop_our_server()
-
         # Proceed with closing...
         result = pyzo.editors.closeAll()
         if not result:
@@ -360,6 +373,13 @@ class MainWindow(QtWidgets.QMainWindow):
             self._closeflag = True
             # event.accept()  # Had to comment on Windows+py3.3 to prevent error
 
+        # Save settings
+        pyzo.saveConfig()
+        pyzo.command_history.save()
+
+        # Stop command server
+        commandline.stop_our_server()
+
         # Proceed with closing shells
         pyzo.localKernelManager.terminateAll()
 
@@ -418,11 +438,17 @@ class MainWindow(QtWidgets.QMainWindow):
             ):
                 args = ["python.exe", "-m", "pyzo"]
 
+            if sys.platform == "win32":
+                # workaround for MSVCRT issue with spaces in arguments
+                #     https://bugs.python.org/issue436259
+                from subprocess import list2cmdline
+
+                args = [list2cmdline([s]) for s in args]
+
             # Replace the process!
             os.execv(sys.executable, args)
 
     def createPopupMenu(self):
-
         # Init menu
         menu = QtWidgets.QMenu()
 
@@ -460,26 +486,32 @@ def loadAppIcons():
     # Qt somehow does not use the highest possible res on Linux/Gnome(?), even
     # the logo of qt-designer when alt-tabbing looks a bit ugly.
     pyzo.icon = QtGui.QIcon()
+
+    # Construct another icon to show when the current shell is busy
+    pyzo.iconRunning = QtGui.QIcon(pyzo.icon)
+
     for sze in [16, 32, 48, 64, 128, 256]:
         fname = os.path.join(appiconDir, fnameT.format(sze))
         if os.path.isfile(fname):
             pyzo.icon.addFile(fname, QtCore.QSize(sze, sze))
 
+            artist = IconArtist(pyzo.icon, size=sze)
+            artist.setPenColor("#0D0")
+            artist.setBrushColor("#0D0")
+            a = 6 * sze // 16
+            x2 = sze - 1
+            x1 = x2 - a + 1
+            y1 = sze - 1
+            y2 = y1 - a + 1
+            y3 = y2 - a + 1
+            artist.addPolygon([(x1, y1), (x2, y2), (x1, y3)])
+            pm = artist.finish().pixmap(sze, sze)
+            pyzo.iconRunning.addPixmap(pm)
+
     # Set as application icon. This one is used as the default for all
     # windows of the application.
     QtWidgets.qApp.setWindowIcon(pyzo.icon)
 
-    # Construct another icon to show when the current shell is busy
-    artist = IconArtist(pyzo.icon)  # extracts the 16x16 version
-    artist.setPenColor("#0B0")
-    for x in range(11, 16):
-        d = x - 11  # runs from 0 to 4
-        artist.addLine(x, 6 + d, x, 15 - d)
-    pm = artist.finish().pixmap(16, 16)
-    #
-    pyzo.iconRunning = QtGui.QIcon(pyzo.icon)
-    pyzo.iconRunning.addPixmap(pm)  # Change only 16x16 icon
-
 
 def loadIcons():
     """loadIcons()
