@@ -114,7 +114,6 @@ class ShellHighlighter(Highlighter):
     """
 
     def highlightBlock(self, line):
-
         # Make sure this is a Unicode Python string
         line = str(line)
 
@@ -181,14 +180,14 @@ class ShellHighlighter(Highlighter):
                             # print(repr(nameToFormat(token.name)))
                             continue
                         # Set format
-                        # format.setFontWeight(75)
+                        # format.setFontWeight(QtGui.QFont.Bold)
                         if token.start >= pos2:
                             self.setFormat(token.start, token.end - token.start, format)
 
             # Set prompt to bold
             if atCurrentPrompt:
                 format = QtGui.QTextCharFormat()
-                format.setFontWeight(75)
+                format.setFontWeight(QtGui.QFont.Bold)
                 self.setFormat(pos1, pos2 - pos1, format)
 
         # Get the indentation setting of the editors
@@ -449,9 +448,7 @@ class BaseShell(BaseTextCtrl):
 
     ## Key handlers
     def keyPressEvent(self, event):
-
         if event.key() in [Qt.Key_Return, Qt.Key_Enter]:
-
             # First check if autocompletion triggered
             if self.potentiallyAutoComplete(event):
                 return
@@ -498,7 +495,6 @@ class BaseShell(BaseTextCtrl):
                 return  # Ignore the key, don't go beyond the prompt
 
         if event.key() in [Qt.Key_Up, Qt.Key_Down] and not self.autocompleteActive():
-
             # needle
             if self._historyNeedle is None:
                 # get partly-written-command
@@ -632,7 +628,6 @@ class BaseShell(BaseTextCtrl):
         self.ensureCursorVisible()
 
     def _handleBackspaces_split(self, text):
-
         # while NOT a backspace at first position, or none found
         i = 9999999999999
         while i > 0:
@@ -660,7 +655,6 @@ class BaseShell(BaseTextCtrl):
         N = 0
 
         for i in range(len(texts)):
-
             # Remove backspaces in text and how many are left
             n, text = self._handleBackspaces_split(texts[i])
             texts[i] = text
@@ -824,8 +818,9 @@ class BaseShell(BaseTextCtrl):
             self._cursor1.removeSelectedText()
             self._cursor2.setPosition(self._cursor1.position(), A_MOVE)
         elif prompt == 2:
-            # Insert text after prompt, inserted text becomes new prompt
-            self._cursor1.setPosition(self._cursor2.position(), A_MOVE)
+            # text becomes new prompt
+            self._cursor1.setPosition(self._cursor2.position(), A_KEEP)
+            self._cursor1.removeSelectedText()
             self._cursor1.setKeepPositionOnInsert(True)
             self._cursor2.setKeepPositionOnInsert(False)
             self._insertText(self._cursor1, text, format)
@@ -872,7 +867,6 @@ class BaseShell(BaseTextCtrl):
         i0 = 0
 
         for match in re.finditer(pattern, text):
-
             # Insert pending text with the current format
             # Also update indices
             i1, i2 = match.span()
@@ -895,16 +889,16 @@ class BaseShell(BaseTextCtrl):
                 if param == 0:
                     format = QtGui.QTextCharFormat()
                 elif param == 1:
-                    format.setFontWeight(75)  # Bold
+                    format.setFontWeight(QtGui.QFont.Bold)
                 elif param == 2:
-                    format.setFontWeight(25)  # Faint
+                    format.setFontWeight(QtGui.QFont.Light)
                 elif param == 3:
                     format.setFontItalic(True)  # Italic
                 elif param == 4:
                     format.setFontUnderline(True)  # Underline
                 #
                 elif param == 22:
-                    format.setFontWeight(50)  # Not bold or faint
+                    format.setFontWeight(QtGui.QFont.Normal)  # Not bold or light
                 elif param == 23:
                     format.setFontItalic(False)  # Not italic
                 elif param == 24:
@@ -926,8 +920,10 @@ class BaseShell(BaseTextCtrl):
             # Process very long text more efficiently.
             # Insert per line (very long lines are split in smaller ones)
             if len(text) > 1024:
+                cursor.beginEditBlock()
                 for line in self._splitLinesForPrinting(text):
                     cursor.insertText(line, format)
+                cursor.endEditBlock()
             else:
                 cursor.insertText(text, format)
 
@@ -1439,14 +1435,15 @@ class PythonShell(BaseShell):
                 prompt = 1
             elif sub is self._strm_prompt:
                 prompt = 2
+                M = M[-1:]  # only use the newest prompt
             # Get color
             color = None
             if sub is self._strm_broker:
-                color = "#000"
+                color = "#fff" if pyzo.darkSyntax else "#000"
             elif sub is self._strm_raw:
-                color = "#888888"  # Halfway
+                color = "#bbb" if pyzo.darkSyntax else "#888888"
             elif sub is self._strm_err:
-                color = "#F00"
+                color = "#f00"
             # Write
             self.write("".join(M), prompt, color)
 
