@@ -57,7 +57,6 @@ class ShellInfo_exe(QtWidgets.QComboBox):
             return "%s  [v%s]" % (p.path, p.version)
 
     def setTheText(self, value):
-
         # Init
         self.clear()
         self.setEditable(True)
@@ -116,7 +115,6 @@ class ShellInfo_ipython(QtWidgets.QCheckBox):
 
 
 class ShellInfo_gui(QtWidgets.QComboBox):
-
     # For (backward) compatibility
     COMPAT = {"QT4": "PYQT4"}
 
@@ -128,7 +126,7 @@ class ShellInfo_gui(QtWidgets.QComboBox):
         ("PySide6", "LGPL licensed wrapper to Qt6"),
         ("PySide2", "LGPL licensed wrapper to Qt5"),
         ("PySide", "LGPL licensed wrapper to Qt4"),
-        ("PyQt6", "GPL/commercial licensed wrapper to Qt5"),
+        ("PyQt6", "GPL/commercial licensed wrapper to Qt6"),
         ("PyQt5", "GPL/commercial licensed wrapper to Qt5"),
         ("PyQt4", "GPL/commercial licensed wrapper to Qt4"),
         ("Tornado", "Tornado asynchronous networking library"),
@@ -141,7 +139,6 @@ class ShellInfo_gui(QtWidgets.QComboBox):
     # GUI descriptions
 
     def setTheText(self, value):
-
         # Process value
         value = value.upper()
         value = self.COMPAT.get(value, value)
@@ -164,7 +161,6 @@ class ShellInfo_gui(QtWidgets.QComboBox):
 
 
 class ShellinfoWithSystemDefault(QtWidgets.QVBoxLayout):
-
     DISABLE_SYSTEM_DEFAULT = sys.platform == "darwin"
     SYSTEM_VALUE = ""
 
@@ -202,7 +198,6 @@ class ShellinfoWithSystemDefault(QtWidgets.QVBoxLayout):
             self.setTheText(self._bufferedValue)
 
     def setTheText(self, value):
-
         if self.DISABLE_SYSTEM_DEFAULT:
             # Just set the value
             self._edit.setReadOnly(False)
@@ -230,11 +225,9 @@ class ShellinfoWithSystemDefault(QtWidgets.QVBoxLayout):
 
 
 class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
-
     SYSTEM_VALUE = "$PYTHONPATH"
 
     def __init__(self, parent):
-
         # Create sub-widget
         self._edit = QtWidgets.QTextEdit(parent)
         self._edit.zoomOut(1)
@@ -286,7 +279,6 @@ class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
 
 
 class ShellInfo_startupScript(QtWidgets.QVBoxLayout):
-
     DISABLE_SYSTEM_DEFAULT = sys.platform == "darwin"
     SYSTEM_VALUE = "$PYTHONSTARTUP"
     RUN_AFTER_GUI_TEXT = "# AFTER_GUI - code below runs after integrating the GUI\n"
@@ -442,7 +434,6 @@ class ShellInfo_environ(QtWidgets.QTextEdit):
 
 
 class ShellInfoTab(QtWidgets.QScrollArea):
-
     INFO_KEYS = [
         translate("shell", "name ::: The name of this configuration."),
         translate("shell", "exe ::: The Python executable."),
@@ -496,19 +487,28 @@ class ShellInfoTab(QtWidgets.QScrollArea):
             # Add to layout
             self._formLayout.addRow(label, instance)
 
-        # Add delete button
+        buttonsLayout = QtWidgets.QHBoxLayout()
 
+        # Add delete button
         t = translate("shell", "Delete ::: Delete this shell configuration")
-        label = QtWidgets.QLabel("", self._content)
         instance = QtWidgets.QPushButton(pyzo.icons.cancel, t, self._content)
         instance.setToolTip(t.tt)
         instance.setAutoDefault(False)
         instance.clicked.connect(self.parent().parent().onTabClose)
-        deleteLayout = QtWidgets.QHBoxLayout()
-        deleteLayout.addWidget(instance, 0)
-        deleteLayout.addStretch(1)
+        buttonsLayout.addWidget(instance, 0)
+
+        # Add copy button
+        t = translate("shell", "Copy ::: Copy this shell configuration")
+        instance = QtWidgets.QPushButton(pyzo.icons.page_white_copy, t, self._content)
+        instance.setToolTip(t.tt)
+        instance.setAutoDefault(False)
+        instance.clicked.connect(self.parent().parent().onTabCopy)
+        buttonsLayout.addWidget(instance, 0)
+
+        buttonsLayout.addStretch(1)
         # Add to layout
-        self._formLayout.addRow(label, deleteLayout)
+        label = QtWidgets.QLabel("", self._content)
+        self._formLayout.addRow(label, buttonsLayout)
 
         # Apply layout
         self._formLayout.setSpacing(15)
@@ -547,7 +547,6 @@ class ShellInfoTab(QtWidgets.QScrollArea):
             print(info)
 
     def getInfo(self):
-
         info = self._info
 
         # Set struct values according to widgets
@@ -645,6 +644,20 @@ class ShellInfoDialog(QtWidgets.QDialog):
         index = self._tabs.currentIndex()
         self._tabs.removeTab(index)
 
+    def onTabCopy(self):
+        # Get original widget
+        w0 = self._tabs.currentWidget()
+        # Build new info
+        info = w0.getInfo().copy()
+        info.name += " (2)"
+        # Create widget and add to tabs
+        w = ShellInfoTab(self._tabs)
+        self._tabs.insertTab(self._tabs.indexOf(w0) + 1, w, "---")
+        w.setInfo(info)
+        # Select
+        self._tabs.setCurrentWidget(w)
+        w.setFocus()
+
     def applyAndClose(self, event=None):
         self.apply()
         self.close()
