@@ -70,8 +70,8 @@ class TitledWidget(QtWidgets.QWidget):
 
         self.setLayout(layout)
 
-    def setFocus(self, val):
-        self.widget.setFocus(val)
+    def setFocus(self):
+        self.widget.setFocus()
 
 
 class ColorLineEdit(QtWidgets.QLineEdit):
@@ -182,9 +182,8 @@ class StyleEdit(QtWidgets.QWidget):
         """
 
         checkBox = QtWidgets.QCheckBox()
-
         self.setters[key] = lambda val, check=checkBox: check.setCheckState(
-            val == "yes"
+            [QtCore.Qt.CheckState.Unchecked, QtCore.Qt.CheckState.Checked][val == "yes"]
         )
 
         checkBox.stateChanged.connect(
@@ -194,7 +193,6 @@ class StyleEdit(QtWidgets.QWidget):
         self.layout.addWidget(TitledWidget(name, checkBox))
 
     def __add_comboBox(self, key, name, *items):
-
         """this is a helper method to create a comboBox
         it adds the created widget (as a TitledWidget) to the layout and
         register a setter and listen to changes
@@ -222,8 +220,8 @@ class StyleEdit(QtWidgets.QWidget):
         for key, setter in self.setters.items():
             setter(style[key])
 
-    def setFocus(self, val):
-        self.layout.itemAt(0).widget().setFocus(True)
+    def setFocus(self):
+        self.layout.itemAt(0).widget().setFocus()
 
 
 class ThemeEditorWidget(QtWidgets.QWidget):
@@ -437,6 +435,7 @@ class ThemeEditorWidget(QtWidgets.QWidget):
 
         if prev != new:
             pyzo.config.settings.theme = new
+            pyzo.saveConfig()
             # This may be better
             pyzo.main.restart()
         else:
@@ -448,7 +447,7 @@ class ThemeEditorWidget(QtWidgets.QWidget):
             self.createTheme()
 
     def focusOnStyle(self, key):
-        self.styleEdits[key].setFocus(True)
+        self.styleEdits[key].setFocus()
         self.scrollArea.ensureWidgetVisible(self.styleEdits[key])
 
     def updatedStyle(self, style, text):
