@@ -89,7 +89,6 @@ class App_base:
         # and then proceed. The inner while-loop is the actual event loop.
         while True:
             try:
-
                 while True:
                     time.sleep(sleep_time)
                     self.process_events()
@@ -116,7 +115,7 @@ class App_nogui(App_base):
 
     def run(self, repl_callback):
         # Move at a slow pace - there is no gui to keep running
-        super().run(repl_callback, 0.1)
+        App_base.run(self, repl_callback, 0.1)
 
 
 # Experimental and WIP - not used at the moment
@@ -268,7 +267,6 @@ class App_tk(App_base):
     """
 
     def __init__(self):
-
         # Try importing
         import sys
 
@@ -469,20 +467,16 @@ class App_qt(App_base):
             """
 
             def __new__(cls, *args, **kwargs):
-
                 # Get the singleton application instance
-                theApp = QApplication_hijacked.instance()
+                theApp = cls.instance()
 
                 # Instantiate an original QApplication instance if we need to
                 if theApp is None:
                     theApp = QtGui.real_QApplication(*args, **kwargs)
                     QtGui.qApp = theApp
 
-                # Add attributes of cls to the instance to make it
-                # behave as if it were an instance of that class
-                for key in dir(cls):
-                    # Skip all magic methods except __init__
-                    if key.startswith("__") and key != "__init__":
+                for key in ["__init__", "exec_", "quit"]:
+                    if not hasattr(cls, key):
                         continue
                     # Skip attributes that we already have
                     val = getattr(cls, key)
@@ -669,7 +663,6 @@ class App_wx(App_base):
     """Hijack the wxWidgets mainloop."""
 
     def __init__(self):
-
         # Try importing
         try:
             import wx
