@@ -35,7 +35,7 @@ import traceback
 import keyword
 import inspect  # noqa - Must be in this namespace
 import bdb
-from distutils.version import LooseVersion as LV
+import linecache
 
 import yoton
 from pyzokernel import guiintegration, printDirect
@@ -131,7 +131,6 @@ class PyzoInterpreter:
     # - support for magic commands
 
     def __init__(self, locals, filename="<console>"):
-
         # Init variables for locals and globals (globals only for debugging)
         self.locals = locals
         self.globals = None
@@ -174,6 +173,7 @@ class PyzoInterpreter:
         # code is expected.
         self.more = 0
         self.newPrompt = True
+        self._oldPromptString = None
 
         # Code and script to run on first iteration
         self._codeToRunOnStartup = None
@@ -354,10 +354,23 @@ class PyzoInterpreter:
 
         # Prevent app nap on OSX 9.2 and up
         # The _nope module is taken from MINRK's appnope package
-        if sys.platform == "darwin" and LV(platform.mac_ver()[0]) >= LV("10.9"):
-            from pyzokernel import _nope
+        if sys.platform == "darwin":
 
-            _nope.nope()
+            def parse_version_crudely(version_string):
+                """extracts the leading number parts of a version string to a tuple
+                e.g.: "123.45ew6.7x.dev8" --> (123, 45, 7)
+                """
+                import re
+
+                return tuple(
+                    int(s) for s in re.findall(r"\.(\d+)", "." + version_string)
+                )
+
+            parsev = parse_version_crudely
+            if parsev(platform.mac_ver()[0]) >= parsev("10.9"):
+                from pyzokernel import _nope
+
+                _nope.nope()
 
         # Setup post-mortem debugging via appropriately logged exceptions
         class PMHandler(logging.Handler):
@@ -611,7 +624,6 @@ class PyzoInterpreter:
     def process_commands(self):
         """Do one iteration of processing commands (the REPL)."""
         try:
-
             self._process_commands()
 
         except SystemExit:
@@ -629,13 +641,12 @@ class PyzoInterpreter:
             self.guiApp.quit()  # Is sys.exit() by default
 
     def _process_commands(self):
-
         # Run startup code/script inside the loop (only the first time)
         # so that keyboard interrupt will work
         if self._codeToRunOnStartup:
             self.context._stat_interpreter.send("Busy")
             self._codeToRunOnStartup, tmp = None, self._codeToRunOnStartup
-            self.pushline(tmp)
+            self._runlines(tmp, filename="<startup_after_gui>", symbol="exec")
         if self._scriptToRunOnStartup:
             self.context._stat_interpreter.send("Busy")
             self._scriptToRunOnStartup, tmp = None, self._scriptToRunOnStartup
@@ -647,10 +658,14 @@ class PyzoInterpreter:
 
         # Set status and prompt?
         # Prompt is allowed to be an object with __str__ method
-        if self.newPrompt:
+        # We also compare prompt strings because stack frame changes
+        # caused by a breakpoint while running the gui event loop
+        # would get unnotified otherwise.
+        promptString = str(sys.ps2 if self.more else sys.ps1)
+        if self.newPrompt or promptString != self._oldPromptString:
             self.newPrompt = False
-            ps = [sys.ps1, sys.ps2][bool(self.more)]
-            self.context._strm_prompt.send(str(ps))
+            self._oldPromptString = promptString
+            self.context._strm_prompt.send(promptString)
 
         if True:
             # Determine state. The message is really only send
@@ -912,9 +927,10 @@ class PyzoInterpreter:
 
         # Get text (make sure it ends with a newline)
         try:
-            bb = open(fname, "rb").read()
+            with open(fname, "rb") as fd:
+                bb = fd.read()
             encoding = "UTF-8"
-            firstline = bb.split("\n".encode(), 1)[0].decode("ascii", "ignore")
+            firstline = bb.split(b"\n", 1)[0].decode("ascii", "ignore")
             if firstline.startswith("#") and "coding" in firstline:
                 encoding = firstline.split("coding", 1)[-1].strip(" \t\r\n:=-*")
             source = bb.decode(encoding)
@@ -1230,22 +1246,22 @@ class ExecutedSourceCollection:
 
     def _patch(self):
         def getlines(filename, module_globals=None):
+            # !!! do not use "import" inside this function because it can
+            # cause an infinite recursion loop with the import override in
+            # module shiboken6 (PySide6) !!!
+
             # Try getting the source from our own cache,
             # otherwise fallback to linecache's own cache
             src = self._cache.get(filename, "")
             if src:
                 return [line + "\n" for line in src.splitlines()]
             else:
-                import linecache
-
                 if module_globals is None:
                     return linecache._getlines(filename)  # only valid sig in 2.4
                 else:
                     return linecache._getlines(filename, module_globals)
 
         # Monkey patch
-        import linecache
-
         linecache._getlines = linecache.getlines
         linecache.getlines = getlines
 
