@@ -75,7 +75,6 @@ class Browser(QtWidgets.QWidget):
         try:
             return self._importWizard
         except AttributeError:
-
             from .importwizard import ImportWizard
 
             self._importWizard = ImportWizard()
@@ -111,8 +110,15 @@ class Browser(QtWidgets.QWidget):
             "matchCase": self.config.searchMatchCase,
             "regExp": self.config.searchRegExp,
             "subDirs": self.config.searchSubDirs,
+            "excludeBinary": self.config.searchExcludeBinary,
         }
 
+    def setSearchText(self, needle, setFocus=False):
+        """Set the text in the search field."""
+        self._searchFilter.setText(needle)
+        if setFocus:
+            self._searchFilter.setFocus()
+
     @property
     def expandedDirs(self):
         """The list of the expanded directories."""
@@ -300,9 +306,13 @@ class PathInput(LineEditWithToolButtons):
         c.setCompletionMode(c.InlineCompletion)
 
         # Set dir model to completer
-        dirModel = QtWidgets.QFileSystemModel(c)
-        dirModel.setFilter(QtCore.QDir.Dirs | QtCore.QDir.NoDotAndDotDot)
-        c.setModel(dirModel)
+        self._dirModel = QtWidgets.QFileSystemModel(c)
+        self._dirModel.setFilter(QtCore.QDir.Dirs | QtCore.QDir.NoDotAndDotDot)
+        # filter is not synchronized with NameFilter input (e.g. "!hidden")
+        c.setModel(self._dirModel)
+        if sys.platform != "win32":
+            self._dirModel.setRootPath("/")
+        # for win32 setRootPath is done self.setPath(...)
 
         # Connect signals
         # c.activated.connect(self.onActivated)
@@ -312,6 +322,11 @@ class PathInput(LineEditWithToolButtons):
 
     def setPath(self, path):
         """Set the path to display. Does nothing if this widget has focus."""
+        if sys.platform == "win32":
+            oldDrive, _ = op.splitdrive(self.text())
+            newDrive, _ = op.splitdrive(path)
+            if oldDrive != newDrive:
+                self._dirModel.setRootPath(newDrive)
         if not self.hasFocus():
             self.setText(path)
             self.checkValid()  # Reset style if it was invalid first
@@ -360,7 +375,6 @@ class PathInput(LineEditWithToolButtons):
 
 
 class Projects(QtWidgets.QWidget):
-
     dirChanged = QtCore.Signal(str)  # Emitted when the user changes the project
 
     def __init__(self, parent):
@@ -643,6 +657,7 @@ class SearchFilter(LineEditWithToolButtons):
             ("searchMatchCase", False, translate("filebrowser", "Match case")),
             ("searchRegExp", False, translate("filebrowser", "RegExp")),
             ("searchSubDirs", True, translate("filebrowser", "Search in subdirs")),
+            ("searchExcludeBinary", True, translate("filebrowser", "Exclude binary")),
         ]
 
         # Fill menu
