@@ -47,21 +47,29 @@ def addIconOverlays(icon, *overlays, offset=(8, 0), overlay_offset=(0, 0)):
     return QtGui.QIcon(pm0)
 
 
-def _filterFileByName(basename, filters):
-
+def _filterFileByName(path, filters, base_path):
     # Init default; return True if there are no filters
     default = True
+    matchCase = False
 
     for filter in filters:
         # Process filters in order
+        if "/" in filter:
+            filename = op.relpath(path, base_path).replace("\\", "/")
+            filename = "/" + filename.lstrip("/")
+        else:
+            filename = op.basename(path)
+        if not matchCase:
+            filter = filter.lower()
+            filename = filename.lower()
         if filter.startswith("!"):
-            # If the filename matches a filter starting with !, hide it
-            if fnmatch.fnmatch(basename, filter[1:]):
+            # If the filename matches an exclusive filter, hide it
+            if fnmatch.fnmatch(filename, filter[1:]):
                 return False
             default = True
         else:
             # If the file name matches a filter not starting with!, show it
-            if fnmatch.fnmatch(basename, filter):
+            if fnmatch.fnmatch(filename, filter):
                 return True
             default = False
 
@@ -120,7 +128,7 @@ def createItemsFun(browser, parent):
                 continue  # Skip hidden files
             if hideHidden and hasHiddenAttribute(entry):
                 continue  # Skip hidden files on Windows
-            if not _filterFileByName(op.basename(entry), nameFilters):
+            if not _filterFileByName(entry, nameFilters, browser._tree.path()):
                 continue
             files.append(entry)
 
@@ -135,7 +143,6 @@ def createItemsFun(browser, parent):
     files.sort(key=filename2sortkey)
 
     if not searchFilter:
-
         # Create dirs
         for path in dirs:
             starred = op.normcase(path) in starredDirs
@@ -152,7 +159,6 @@ def createItemsFun(browser, parent):
             item = FileItem(parent, fsProxy.file(path))
 
     else:
-
         # If searching, inject everything in the tree
         # And every item is hidden at first
         parent = browser._tree
@@ -352,8 +358,8 @@ class FileItem(BrowserItem):
         if not op.isfile(dummy_filename):
             if not isdir(op.dirname(dummy_filename)):
                 os.makedirs(op.dirname(dummy_filename))
-            f = open(dummy_filename, "wb")
-            f.close()
+            with open(dummy_filename, "wb"):
+                pass
         # Use that file
         if sys.platform.startswith("linux") and not QtCore.__file__.startswith("/usr/"):
             icon = iconprovider.icon(iconprovider.File)
@@ -402,7 +408,6 @@ class FileItem(BrowserItem):
         pass
 
     def onTaskFinished(self, task):
-
         if isinstance(task, tasks.DocstringTask):
             result = task.result()
             self.clear()  # Docstring task is done *before* peek task
@@ -627,10 +632,19 @@ class Tree(QtWidgets.QTreeWidget):
         # Close old proxy
         if self._proxy is not None:
             self._proxy.cancel()
-            self._proxy.changed.disconnect(self.onChanged)
-            self._proxy.deleted.disconnect(self.onDeleted)
-            self._proxy.errored.disconnect(self.onErrored)
-            self.destroyed.disconnect(self._proxy.cancel)
+            failed = []
+            for signal, func in [
+                (self.destroyed, self._proxy.cancel),
+                (self._proxy.changed, self.onChanged),
+                (self._proxy.deleted, self.onDeleted),
+                (self._proxy.errored, self.onErrored),
+            ]:
+                try:
+                    signal.disconnect(func)
+                except Exception:
+                    failed.append(signal)
+            if failed:
+                pass  # print("Failed to disconnect", failed)
         # Create new proxy
         if True:
             self._proxy = self.parent()._fsProxy.dir(path)
@@ -678,7 +692,7 @@ class Tree(QtWidgets.QTreeWidget):
         QtWidgets.QTreeWidget.clear(self)
 
     def mouseDoubleClickEvent(self, event):
-        """Bypass expanding an item when double-cliking it.
+        """Bypass expanding an item when double-clicking it.
         Only activate the item.
         """
         item = self.itemAt(event.x(), event.y())
@@ -778,7 +792,6 @@ class PopupMenu(pyzo.core.menu.Menu):
         pyzo.core.menu.Menu.__init__(self, parent, " ")
 
     def build(self):
-
         isplat = sys.platform.startswith
 
         # The star object
@@ -924,7 +937,6 @@ class PopupMenu(pyzo.core.menu.Menu):
         self._createDirOrFile(False)
 
     def _createDirOrFile(self, file=True):
-
         # Get title and label
         if file:
             title = translate("filebrowser", "Create new file")
@@ -951,7 +963,6 @@ class PopupMenu(pyzo.core.menu.Menu):
             self._item._proxy.pushTask(task)
 
     def _duplicateOrRename(self, rename):
-
         # Get dirname and filename
         dirname, filename = op.split(self._item.path())
 
