@@ -302,7 +302,7 @@ htmlWrap = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN"
 <html>
 <head>
 <style type="text/css">
-pre, code {{background-color: #F2F2F2;}}
+pre, code {{background-color: BGCOLOR_CODE;}}
 </style>
 </head>
 <body style=" font-family:'Sans Serif'; font-size:{}pt; font-weight:400; font-style:normal;">
@@ -311,9 +311,11 @@ pre, code {{background-color: #F2F2F2;}}
 </html>
 """
 
+
 # Define title text (font-size percentage does not seem to work sadly.)
 def get_title_text(objectName, h_class="", h_repr=""):
-    title_text = "<p style='background-color:#def;'>"
+    title_bgcolor = "#468" if pyzo.darkQt else "#def"
+    title_text = "<p style='background-color:{};'>".format(title_bgcolor)
     if h_class == "~python_keyword~":
         title_text += "<b>Keyword:</b> {}".format(objectName)
     elif h_class == "~python_operator~":
@@ -522,7 +524,6 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
             self.setText()
 
     def setText(self, text=None):
-
         # (Re)store text
         if text is None:
             text = self._browser_text
@@ -531,7 +532,11 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
 
         # Set text with html header
         size = self._config.fontSize
-        self._browser.setHtml(htmlWrap.format(size, text))
+        self._browser.setHtml(
+            htmlWrap.replace(
+                "BGCOLOR_CODE", "#505050" if pyzo.darkQt else "#f2f2f2"
+            ).format(size, text)
+        )
 
     def setObjectName(self, name, addToHist=False):
         """Set the object name programatically
@@ -651,7 +656,6 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
             h_text = h_text.replace(">", "&gt;")
 
             if self._config.smartNewlines:
-
                 # Make sure the signature is separated from the rest using at
                 # least two newlines
                 header = ""
@@ -660,7 +664,9 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
                     name = objectName.split(".")[-1]
                     # Is the signature in the docstring?
                     docs = h_text.replace("\n", "|")
-                    tmp = re.search("[a-zA-z_\.]*?" + name + "\(.*?\)", docs)
+                    tmp = re.search(
+                        r"[a-zA-z_\.]*?" + re.escape(name) + r"\(.*?\)", docs
+                    )
                     if tmp and tmp.span(0)[0] < 5:
                         header = tmp.group(0)
                         h_text = h_text[len(header) :].lstrip(":").lstrip()
@@ -675,9 +681,10 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
 
                 # Parse the text as rest/numpy like docstring
                 h_text = self.smartFormat(h_text)
-                h_text = re.sub("``(.*?)``", r"<code>\1</code>", h_text)
+                h_text = re.sub(r"``(.*?)``", r"<code>\1</code>", h_text)
                 if header:
-                    h_text = "<p style='color:#005;'><b>%s</b></p>\n%s" % (
+                    h_text = "<p style='color:{};'><b>{}</b></p>\n{}".format(
+                        "#0bf" if pyzo.darkQt else "#005",
                         header,
                         h_text,
                     )
@@ -706,7 +713,6 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
         self.setText(text)
 
     def smartFormat(self, text):
-
         # Get lines
         lines = text.splitlines(True)
 
@@ -734,7 +740,6 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
         # Format line by line
         lines3 = []
         for line in lines2:
-
             # Get indentation
             line_ = line.lstrip()
             if line_ in ("```", "```\n"):
