@@ -12,7 +12,6 @@ import os
 
 from .pythoninterpreter import EXE_DIR, PythonInterpreter, versionStringToTuple
 from .inwinreg import get_interpreters_in_reg
-from .. import paths
 
 
 def get_interpreters(minimumVersion=None):
@@ -35,14 +34,11 @@ def get_interpreters(minimumVersion=None):
     # Get relative interpreters
     relative = set([PythonInterpreter(p) for p in _get_interpreters_relative()])
 
-    # Get Pyzo paths
-    pyzos = set([PythonInterpreter(p) for p in _get_interpreters_pyzo()])
-
     # Get pipenv paths
     pipenvs = set([PythonInterpreter(p) for p in _get_interpreters_pipenv()])
 
     # Almost done
-    interpreters = set.union(pythons, condas, relative, pyzos, pipenvs)
+    interpreters = set.union(pythons, condas, relative, pipenvs)
     minimumVersion = minimumVersion or "0"
     return _select_interpreters(interpreters, minimumVersion)
 
@@ -112,6 +108,15 @@ def _get_interpreters_win():
 def _get_interpreters_posix():
     found = []
 
+    def isPathValidPythonExe(filename):
+        fname = os.path.split(filename)[1]
+        return (
+            fname.startswith(("python", "pypy"))
+            and not fname.count("config")
+            and len(fname) < 16
+            and os.path.isfile(filename)
+        )
+
     # Look for system Python interpreters
     for searchpath in ["/usr/bin", "/usr/local/bin", "/opt/local/bin"]:
         searchpath = os.path.expanduser(searchpath)
@@ -124,14 +129,12 @@ def _get_interpreters_posix():
 
         # Search for python executables
         for fname in files:
-            if fname.startswith(("python", "pypy")) and not fname.count("config"):
-                if len(fname) < 16:
-                    # Get filename and resolve symlink
-                    filename = os.path.join(searchpath, fname)
-                    filename = os.path.realpath(filename)
-                    # Seen on OS X that was not a valid file
-                    if os.path.isfile(filename):
-                        found.append(filename)
+            # Get filename and resolve symlink
+            # Seen on OS X that was not a valid file
+            filename = os.path.join(searchpath, fname)
+            filename = os.path.realpath(filename)
+            if isPathValidPythonExe(filename):
+                found.append(filename)
 
     # Look for user-installed Python interpreters such as pypy and anaconda
     for rootname in ["~", "/usr/local"]:
@@ -145,6 +148,13 @@ def _get_interpreters_posix():
                     if os.path.isfile(exename):
                         found.append(exename)
 
+    # Look for Python interpreter provided by PYZO_DEFAULT_SHELL_PYTHON_EXE env-var
+    if "PYZO_DEFAULT_SHELL_PYTHON_EXE" in os.environ:
+        filename = os.environ["PYZO_DEFAULT_SHELL_PYTHON_EXE"]
+        filename = os.path.realpath(filename)
+        if isPathValidPythonExe(filename):
+            found.append(filename)
+
     # Remove pythonw, pythonm and the like
     found = set(found)
     for path in list(found):
@@ -155,21 +165,6 @@ def _get_interpreters_posix():
     return set(found)
 
 
-def _get_interpreters_pyzo():
-    """Get a list of known Pyzo interpreters."""
-    pythonname = "python" + ".exe" * sys.platform.startswith("win")
-    exes = []
-    for d in paths.pyzo_dirs():
-        for fname in [
-            os.path.join(d, "bin", pythonname + "3"),
-            os.path.join(d, pythonname),
-        ]:
-            if os.path.isfile(fname):
-                exes.append(fname)
-                break
-    return exes
-
-
 def _get_interpreters_conda():
     """Get known conda environments"""
     if sys.platform.startswith("win"):
@@ -180,11 +175,12 @@ def _get_interpreters_conda():
     exes = []
     filename = os.path.expanduser("~/.conda/environments.txt")
     if os.path.isfile(filename):
-        for line in open(filename, "rt").readlines():
-            line = line.strip()
-            exe_filename = os.path.join(line, pythonname)
-            if line and os.path.isfile(exe_filename):
-                exes.append(exe_filename)
+        with open(filename, "rt", encoding="utf-8") as fd:
+            for line in fd.readlines():
+                line = line.strip()
+                exe_filename = os.path.join(line, pythonname)
+                if line and os.path.isfile(exe_filename):
+                    exes.append(exe_filename)
     return exes
 
 
