@@ -10,6 +10,7 @@ Defines the channel classes for the req/rep pattern.
 
 """
 
+import sys
 import time
 import threading
 
@@ -58,7 +59,6 @@ class Future(object):
     """
 
     def __init__(self, req_channel, req, request_id):
-
         # For being a Future object
         self._result = None
         self._status = 0  # 0:waiting, 1:running, 2:canceled, 3:error, 4:success
@@ -726,7 +726,6 @@ class RepChannel(BaseChannel):
         return func(*args, **kwargs)
 
     def _acknowledge_next_pre_request(self):
-
         # Cancel current pre-request ourselves if it takes too long.
         # Failsafe, only for if resetting by requester fails somehow.
         if time.time() - self._pre_request_time > 10.0:
@@ -734,7 +733,6 @@ class RepChannel(BaseChannel):
 
         # Send any pending pre requests
         if self._pre_requests and not self._pre_request:
-
             # Set current pre-request and its ack time
             package = self._pre_requests.pop(0)
             self._pre_request = package
@@ -870,8 +868,11 @@ class ThreadForReqChannel(threading.Thread):
         # Store channel
         self._channel = channel
 
-        # Make deamon
-        self.setDaemon(True)
+        # Make daemon
+        if sys.version_info < (2, 6):
+            self.setDaemon(True)
+        else:
+            self.daemon = True
 
     def run(self):
         """run()
@@ -885,7 +886,6 @@ class ThreadForReqChannel(threading.Thread):
         del self._channel
 
         while True:
-
             # Stop?
             if channel.closed or channel._run_mode != 2:
                 break
