@@ -33,6 +33,7 @@ servers in a single process, that listen on different ports.
 
 """
 
+import sys
 import time
 import socket
 import threading
@@ -96,8 +97,12 @@ class RequestServer(threading.Thread):
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 
-        # Bind (can raise error is port is not available)
-        s.bind((host, port))
+        # Bind (can raise error if port is not available)
+        try:
+            s.bind((host, port))
+        except Exception:
+            s.close()
+            raise
 
         # Store socket instance
         self._bsd_socket = s
@@ -105,8 +110,11 @@ class RequestServer(threading.Thread):
         # To stop serving
         self._stop_me = False
 
-        # Make deamon
-        self.setDaemon(True)
+        # Make daemon
+        if sys.version_info < (2, 6):
+            self.setDaemon(True)
+        else:
+            self.daemon = True
 
     def start(self):
         """start()
