@@ -55,7 +55,6 @@ class TcpConnection(Connection):
     """
 
     def __init__(self, context, name=""):
-
         # Variables to hold threads
         self._sendingThread = None
         self._receivingThread = None
@@ -95,7 +94,6 @@ class TcpConnection(Connection):
         Connection._set_status(self, status)
 
         try:
-
             if status in [STATUS_HOSTING, STATUS_CONNECTED]:
                 # Really connected
 
@@ -114,7 +112,6 @@ class TcpConnection(Connection):
                 self._receivingThread.start()
 
             if status == 0:
-
                 # Close bsd socket
                 try:
                     self._bsd_socket.shutdown()
@@ -161,9 +158,10 @@ class TcpConnection(Connection):
             s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 
         # Try all ports in the specified range
-        for port2 in range(port, port + max_tries):
+        for try_count in range(1, max_tries + 1):
+            port += int(try_count**0.5)
             try:
-                s.bind((hostname, port2))
+                s.bind((hostname, port))
                 break
             except Exception:
                 # Raise the socket exception if we were asked to try
@@ -173,9 +171,8 @@ class TcpConnection(Connection):
                 continue
         else:
             # We tried all ports without success
-            tmp = str(max_tries)
-            tmp = "Could not bind to any of the " + tmp + " ports tried."
-            raise IOError(tmp)
+            tmp = "Could not bind to any of the %i %i ports tried."
+            raise IOError(tmp % (max_tries, port))
 
         # Tell the socket it is a host. Backlog of at least 1 to avoid linux
         # kernel from detecting SYN flood and throttling the connection (#381)
@@ -299,8 +296,11 @@ class HostThread(threading.Thread):
         self._context_connection = context_connection
         self._bsd_host_socket = bsd_socket
 
-        # Make deamon
-        self.setDaemon(True)
+        # Make daemon
+        if sys.version_info < (2, 6):
+            self.setDaemon(True)
+        else:
+            self.daemon = True
 
     def run(self):
         """run()
@@ -312,7 +312,6 @@ class HostThread(threading.Thread):
 
         # Try making a connection until success or the context is stopped
         while self._context_connection.is_waiting:
-
             # Wait for connection
             s = self._wait_for_connection()
             if not s:
@@ -388,7 +387,6 @@ class HandShaker:
     """
 
     def __init__(self, bsd_socket):
-
         # Store bsd socket
         self._bsd_socket = bsd_socket
 
@@ -485,7 +483,10 @@ class BaseIOThread(threading.Thread):
         threading.Thread.__init__(self)
 
         # Thread will "destruct" when the interpreter shuts down
-        self.setDaemon(True)
+        if sys.version_info < (2, 6):
+            self.setDaemon(True)
+        else:
+            self.daemon = True
 
         # Store (temporarily) the ref to the context connection
         # Also of bsd_socket, because it might be removed before the
@@ -528,7 +529,6 @@ class BaseIOThread(threading.Thread):
             sys.__stderr__.flush()
 
         try:
-
             # Enter mainloop
             stop_reason = self._run(context_connection, bsd_socket)
 
