@@ -21,6 +21,7 @@ more information. Note that signals only work if events are processed.
 
 """
 
+import sys
 import time
 import threading
 import weakref
@@ -42,7 +43,6 @@ class CallableObject(object):
     __slots__ = ["_ob", "_func"]  # Use __slots__ to reduce memory footprint
 
     def __init__(self, c):
-
         # Check
         if not hasattr(c, "__call__"):
             raise ValueError("Error: given callback is not callable.")
@@ -101,7 +101,10 @@ class CallableObject(object):
         try:
             return func(*args, **kwargs)
         except Exception:
-            print("Exception while handling event:")
+            funcname = self._func
+            if hasattr(funcname, "__name__"):
+                funcname = funcname.__name__
+            print("Exception while handling event (in %s):" % (funcname,))
             print(getErrorMsg())
 
 
@@ -254,7 +257,10 @@ class TheTimerThread(threading.Thread):
 
     def __init__(self):
         threading.Thread.__init__(self)
-        self.setDaemon(True)
+        if sys.version_info < (2, 6):
+            self.setDaemon(True)
+        else:
+            self.daemon = True
         self._exit = False
         self._timers = []
         self._somethingChanged = False
@@ -312,7 +318,6 @@ class TheTimerThread(threading.Thread):
 
     def _mainloop(self):
         while not self._exit:
-
             # Set flag
             self._somethingChanged = False
 
