@@ -2,9 +2,9 @@ import os
 
 
 def count_lines(filename):
-
     # Get text
-    text = open(filename, "r").read()
+    with open(filename, "rt", encoding="utf-8") as fd:
+        text = fd.read()
 
     # Init counts: code, docstring, comments, whitespace
     count1, count2, count3, count4 = 0, 0, 0, 0
@@ -28,7 +28,6 @@ def count_lines(filename):
 
 
 if __name__ == "__main__":
-
     # Get path of yoton
     path = os.path.dirname(os.path.abspath(__file__))
     path = os.path.split(path)[0]
