@@ -29,6 +29,9 @@ import subprocess
 try:
     if sys.executable.lower().endswith("pythonw.exe"):
         raise ImportError("Dont use faulthandler in pythonw.exe")
+    if sys.platform == "win32":
+        raise ImportError("Prevent crash with QFileIconProvider")
+        # see https://github.com/pyzo/pyzo/issues/875 for details
     import faulthandler
 
     faulthandler.enable()
@@ -37,7 +40,6 @@ except ImportError:
 
 
 if "--test" in sys.argv:
-
     # Prepare log file
     logfilename = os.path.abspath(os.path.join(__file__, "..", "log.txt"))
     with open(logfilename, "wt") as f:
@@ -48,7 +50,6 @@ if "--test" in sys.argv:
     subprocess.run([sys.executable, "pyzo", "--test"])
 
 else:
-
     import pyzo
 
     pyzo.start()
